@@ -61,9 +61,13 @@ SPF_server_get_default_explanation(SPF_server_t *spf_server,
 		return err;
 	}
 	else {
-		if (*buflenp < sizeof(SPF_LAME_EXP)) {
-			*buflenp = sizeof(SPF_LAME_EXP);
-			*bufp = realloc(*bufp, *buflenp);
+		size_t	len = sizeof(SPF_LAME_EXP) + 1;
+		if (*buflenp < len) {
+			char	*tmp = realloc(*bufp, len);
+			if (tmp == NULL)
+				return SPF_E_NO_MEMORY;
+			*bufp = tmp;
+			*buflenp = len;
 		}
 		strcpy(*bufp, SPF_LAME_EXP);
 		return SPF_E_SUCCESS;
@@ -152,29 +156,34 @@ SPF_request_get_exp(SPF_server_t *spf_server,
 		return resolver->get_exp(spf_server, *bufp, bufp, buflenp);
 
 	rr_txt = SPF_dns_lookup(resolver, *bufp, ns_t_txt, TRUE);
-
-	switch( rr_txt->herrno )
-	{
-	case HOST_NOT_FOUND:
-	case NO_DATA:
+	if (rr_txt == NULL) {
+		SPF_dns_rr_free(rr_txt);
 		RETURN_DEFAULT_EXP();
-		break;
-
-	case TRY_AGAIN:
-		RETURN_DEFAULT_EXP();
-		break;
-
-	case NETDB_SUCCESS:
-		break;
-
-	default:
-		SPF_warning("unknown DNS lookup error code");
-		RETURN_DEFAULT_EXP();
-		break;
 	}
 
-	if ( rr_txt->num_rr == 0 )
-	{
+	switch (rr_txt->herrno) {
+		case HOST_NOT_FOUND:
+		case NO_DATA:
+			SPF_dns_rr_free(rr_txt);
+			RETURN_DEFAULT_EXP();
+			break;
+
+		case TRY_AGAIN:
+			SPF_dns_rr_free(rr_txt);
+			RETURN_DEFAULT_EXP();
+			break;
+
+		case NETDB_SUCCESS:
+			break;
+
+		default:
+			SPF_warning("Unknown DNS lookup error code");
+			SPF_dns_rr_free(rr_txt);
+			RETURN_DEFAULT_EXP();
+			break;
+	}
+
+	if (rr_txt->num_rr == 0) {
 		SPF_response_add_warn(spf_response, SPF_E_NOT_SPF,
 				"No TXT records returned from DNS lookup");
 		RETURN_DEFAULT_EXP();
@@ -188,9 +197,13 @@ SPF_request_get_exp(SPF_server_t *spf_server,
 	/* FIXME  we are supposed to concatenate the TXT records */
 
 	/* FIXME: If this generates any errors, demote them to warnings. */
+	spf_macro = NULL;
 	err = SPF_record_compile_macro(spf_server, spf_response, &spf_macro,
 								rr_txt->rr[0]->txt);
 	if (err != SPF_E_SUCCESS) {
+		if (spf_macro)
+			SPF_macro_free(spf_macro);
+		SPF_dns_rr_free(rr_txt);
 		RETURN_DEFAULT_EXP();
 	}
 
@@ -199,6 +212,7 @@ SPF_request_get_exp(SPF_server_t *spf_server,
 					SPF_macro_data(spf_macro), spf_macro->macro_len,
 					bufp, buflenp);
 	SPF_macro_free(spf_macro);
+	SPF_dns_rr_free(rr_txt);
 
 	return err;
 }
